"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common = require("./image-cache-it.common");
var image_cache_it_common_1 = require("./image-cache-it.common");
var imageSrc = require("tns-core-modules/image-source");
var view_1 = require("tns-core-modules/ui/core/view");
var fs = require("tns-core-modules/file-system");
var types = require("tns-core-modules/utils/types");
var style_properties_1 = require("tns-core-modules/ui/styling/style-properties");
global.moduleMerge(common, exports);
var main_queue = dispatch_get_current_queue();
var filter_queue = dispatch_get_global_queue(21, 0);
var ImageCacheIt = (function (_super) {
    __extends(ImageCacheIt, _super);
    function ImageCacheIt() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ImageCacheIt.prototype.createNativeView = function () {
        var nativeView = UIImageView.new();
        nativeView.contentMode = 1;
        nativeView.userInteractionEnabled = true;
        nativeView.clipsToBounds = true;
        var metalDevice = MTLCreateSystemDefaultDevice() || null;
        if (metalDevice) {
            this.ctx = CIContext.contextWithMTLDevice(metalDevice);
        }
        else {
            this.ctx = new CIContext(null);
        }
        return nativeView;
    };
    ImageCacheIt.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var nativeView = this.nativeView;
        if (nativeView) {
            var width = view_1.layout.getMeasureSpecSize(widthMeasureSpec);
            var height = view_1.layout.getMeasureSpecSize(heightMeasureSpec);
            this.setMeasuredDimension(width, height);
        }
    };
    ImageCacheIt.prototype._loadImage = function (src) {
        var _this = this;
        if (this.nativeView) {
            this.nativeView.sd_cancelCurrentImageLoad();
        }
        if (!types.isNullOrUndefined(src)) {
            if (types.isString(src) && src.startsWith('http')) {
                var options = 1024 | 1 | 2048;
                this.isLoading = true;
                var context = {};
                var placeholder = this.placeHolder
                    ? imageSrc.fromFileOrResource(this.placeHolder).ios
                    : null;
                this.nativeView.sd_setImageWithURLPlaceholderImageOptionsContextProgressCompleted(src, placeholder, options, context, function (p1, p2, p3) {
                }, function (p1, p2, p3, p4) {
                    _this.isLoading = false;
                    if (_this.filter) {
                    }
                    if (p2 && _this.errorHolder) {
                        var source = imageSrc.fromFileOrResource(_this.errorHolder);
                        _this.nativeView.image = source ? source.ios : null;
                        _this.setAspect(_this.stretch);
                    }
                    else if (p3 !== SDImageCacheType.Memory && _this.transition) {
                        switch (_this.transition) {
                            case 'fade':
                                _this.nativeView.alpha = 0;
                                UIView.animateWithDurationAnimations(1, function () {
                                    _this.nativeView.alpha = 1;
                                });
                                break;
                            default:
                                break;
                        }
                    }
                    if (p1) {
                        if (_this.filter) {
                            dispatch_async(filter_queue, function () {
                                _this._setupFilter(p1);
                            });
                        }
                        else {
                            _this._setupFilter(p1);
                        }
                    }
                });
            }
            else if (typeof src === 'string' &&
                (src.startsWith('/') || src.startsWith('file'))) {
                var source = imageSrc.fromFileOrResource(src);
                this._setupFilter(source ? source.ios : null);
            }
            else if (typeof src === 'string' &&
                src.startsWith('~')) {
                var path = fs.knownFolders.currentApp().path;
                var file = fs.path.join(path, src.replace('~', ''));
                var source = imageSrc.fromFileOrResource(file);
                this._setupFilter(source ? source.ios : null);
            }
            else if (typeof src === 'string' && src.startsWith('res://')) {
                this._setupFilter(UIImage.imageNamed(src.replace('res://', '')));
            }
            else if (types.isObject(src) && src.ios) {
                this._setupFilter(src.ios);
            }
            else if (types.isObject(src) && src instanceof UIImage) {
                this._setupFilter(src);
            }
        }
        else {
            if (this.fallback) {
                this._loadImage(this.fallback);
            }
        }
    };
    ImageCacheIt.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._loadImage(this.src);
    };
    ImageCacheIt.prototype[common.srcProperty.getDefault] = function () {
        return undefined;
    };
    ImageCacheIt.prototype[common.srcProperty.setNative] = function (src) {
        this._loadImage(src);
    };
    ImageCacheIt.prototype[common.resizeProperty.setNative] = function (resize) {
        if (!this.nativeView)
            return resize;
        if (this.resize &&
            this.resize.split(',').length > 1) {
            this.nativeView.frame.size.width = parseInt(this.resize.split(' ')[0]);
            this.nativeView.frame.size.height = parseInt(this.resize.split(' ')[1]);
        }
        return resize;
    };
    ImageCacheIt.prototype.setAspect = function (value) {
        if (!this.nativeView)
            return value;
        switch (value) {
            case 'aspectFit':
                this.nativeView.contentMode = 1;
                break;
            case 'aspectFill':
                this.nativeView.contentMode = 2;
                break;
            case 'fill':
                this.nativeView.contentMode = 0;
                break;
            case 'none':
            default:
                this.nativeView.contentMode = 9;
                break;
        }
        return value;
    };
    ImageCacheIt.prototype[common.stretchProperty.getDefault] = function () {
        return 'aspectFit';
    };
    ImageCacheIt.prototype[common.stretchProperty.setNative] = function (value) {
        this.setAspect(value);
    };
    ImageCacheIt.prototype[common.filterProperty.setNative] = function (filter) {
        this.filter = filter;
        this._setupFilter(this.nativeView.image);
    };
    ImageCacheIt.prototype._setupFilter = function (image) {
        var _this = this;
        var getValue = function (value) {
            return value.substring(value.indexOf('(') + 1, value.indexOf(')'));
        };
        var createFilterWithName = function (value) {
            var filter;
            if (!ImageCacheIt.ciFilterMap[value]) {
                ImageCacheIt.ciFilterMap[value] = CIFilter.filterWithName(value);
            }
            filter = ImageCacheIt.ciFilterMap[value];
            filter.setDefaults();
            if (image && image.CIImage) {
                filter.setValueForKey(image.CIImage, kCIInputImageKey);
                filter.setValueForKey(NSNull, kCIImageColorSpace);
            }
            else {
                if (image && image.CGImage) {
                    filter.setValueForKey(CIImage.imageWithCGImage(image.CGImage), kCIInputImageKey);
                }
            }
            return filter;
        };
        if (this.filter) {
            var filters = this.filter ? this.filter.split(' ') : [];
            filters.forEach(function (filter) {
                var value = getValue(filter);
                if (filter.indexOf('blur') > -1) {
                    var width = -1;
                    if (value.indexOf('%') === -1) {
                        value = style_properties_1.Length.parse(value);
                        if (value.unit === 'px') {
                            width = value.value;
                        }
                        else if (value.unit === 'dip') {
                            width = view_1.layout.toDevicePixels(value.unit);
                        }
                        if (width > -1) {
                            var blurFilter = createFilterWithName('CIMotionBlur');
                            blurFilter.setValueForKey(width, kCIInputRadiusKey);
                            var blurredImg = blurFilter.valueForKey(kCIOutputImageKey);
                            if (blurredImg && blurredImg.extent) {
                                var cgiImage_1 = _this.ctx.createCGImageFromRect(blurredImg, blurredImg.extent);
                                dispatch_async(main_queue, function () {
                                    _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_1);
                                    _this.setAspect(_this.stretch);
                                });
                            }
                        }
                    }
                }
                else if (filter.indexOf('contrast') > -1) {
                    if (value.indexOf('%')) {
                        var contrast = parseFloat(value.replace('%', '')) / 100;
                        var contrastFilter = createFilterWithName('CIColorControls');
                        contrastFilter.setValueForKey(contrast, kCIInputContrastKey);
                        var contrastImg = contrastFilter.valueForKey(kCIOutputImageKey);
                        if (contrastImg && contrastImg.extent) {
                            var cgiImage_2 = _this.ctx.createCGImageFromRect(contrastImg, contrastImg.extent);
                            dispatch_async(main_queue, function () {
                                _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_2);
                                _this.setAspect(_this.stretch);
                            });
                        }
                    }
                }
                else if (filter.indexOf('brightness') > -1) {
                    if (value.indexOf('%')) {
                        var brightness = parseFloat(value.replace('%', '')) / 100;
                        var brightnessFilter = createFilterWithName('CIColorControls');
                        brightnessFilter.setValueForKey(brightness, kCIInputContrastKey);
                        var contrastImg = brightnessFilter.valueForKey(kCIOutputImageKey);
                        if (contrastImg && contrastImg.extent) {
                            var cgiImage_3 = _this.ctx.createCGImageFromRect(contrastImg, contrastImg.extent);
                            dispatch_async(main_queue, function () {
                                _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_3);
                                _this.setAspect(_this.stretch);
                            });
                        }
                    }
                }
                else if (filter.indexOf('grayscale') > -1 || filter.indexOf('greyscale') > -1) {
                    var grayscale = 0;
                    if (value.indexOf('%') > -1) {
                        grayscale = parseFloat(value.replace('%', '')) / 100;
                    }
                    else if (value.indexOf('.') > -1) {
                        grayscale = parseFloat(value);
                    }
                    else {
                        grayscale = parseInt(value, 10);
                    }
                    if (grayscale > 1) {
                        grayscale = 1;
                    }
                    grayscale = 1 - grayscale;
                    var grayscaleFilter = createFilterWithName('CIColorControls');
                    grayscaleFilter.setValueForKey(grayscale, kCIInputSaturationKey);
                    var grayscaleImg = grayscaleFilter.valueForKey(kCIOutputImageKey);
                    if (grayscaleImg && grayscaleImg.extent) {
                        var cgiImage_4 = _this.ctx.createCGImageFromRect(grayscaleImg, grayscaleImg.extent);
                        dispatch_async(main_queue, function () {
                            _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_4);
                            _this.setAspect(_this.stretch);
                        });
                    }
                }
                else if (filter.indexOf('invert') > -1) {
                    var invertFilter = createFilterWithName('CIColorInvert');
                    var invertImg = invertFilter.valueForKey(kCIOutputImageKey);
                    if (invertImg && invertImg.extent) {
                        var cgiImage_5 = _this.ctx.createCGImageFromRect(invertImg, invertImg.extent);
                        dispatch_async(main_queue, function () {
                            _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_5);
                            _this.setAspect(_this.stretch);
                        });
                    }
                }
                else if (filter.indexOf('sepia') > -1) {
                    var sepia = parseFloat(value.replace('%', '')) / 100;
                    var sepiaFilter = createFilterWithName('CISepiaTone');
                    sepiaFilter.setValueForKey(sepia, kCIInputIntensityKey);
                    var sepiaImg = sepiaFilter.valueForKey(kCIOutputImageKey);
                    if (sepiaImg && sepiaImg.extent) {
                        var cgiImage_6 = _this.ctx.createCGImageFromRect(sepiaImg, sepiaImg.extent);
                        dispatch_async(main_queue, function () {
                            _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_6);
                            _this.setAspect(_this.stretch);
                        });
                    }
                }
                else if (filter.indexOf('opacity') > -1) {
                    var alpha_1 = 1.0;
                    if (value.indexOf('%') > -1) {
                        alpha_1 = parseInt(value.replace('%', ''), 10) / 100;
                    }
                    else if (value.indexOf('.') > -1) {
                        alpha_1 = parseFloat(value);
                    }
                    else {
                        alpha_1 = parseInt(value, 10);
                    }
                    dispatch_async(main_queue, function () {
                        _this.nativeView.alpha = alpha_1;
                        _this.setAspect(_this.stretch);
                    });
                }
                else if (filter.indexOf('hue') > -1) {
                    var hueFilter = createFilterWithName('CIHueAdjust');
                    var hue = 0;
                    if (value.indexOf('deg') > -1) {
                        hue = parseInt(value.replace('deg', ''), 10);
                    }
                    else if (value.indexOf('turn') > -1) {
                        hue = parseInt(value.replace('turn', ''), 10) * 360;
                    }
                    hueFilter.setValueForKey(hue, kCIInputAngleKey);
                    var hueImg = hueFilter.valueForKey(kCIOutputImageKey);
                    if (hueImg && hueImg.extent) {
                        var cgiImage_7 = _this.ctx.createCGImageFromRect(hueImg, hueImg.extent);
                        dispatch_async(main_queue, function () {
                            _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_7);
                            _this.setAspect(_this.stretch);
                        });
                    }
                }
                else if (filter.indexOf('saturate') > -1) {
                    var saturateFilter = createFilterWithName('CIColorControls');
                    var saturate = 1.0;
                    if (value.indexOf('%') > -1) {
                        saturate = parseInt(value.replace('%', ''), 10) / 100;
                    }
                    else if (value.indexOf('.') > -1) {
                        saturate = parseFloat(value);
                    }
                    else {
                        saturate = parseInt(value, 10);
                    }
                    saturateFilter.setValueForKey(saturate, kCIInputSaturationKey);
                    var saturateImg = saturateFilter.valueForKey(kCIOutputImageKey);
                    if (saturateImg && saturateImg.extent) {
                        var cgiImage_8 = _this.ctx.createCGImageFromRect(saturateImg, saturateImg.extent);
                        dispatch_async(main_queue, function () {
                            _this.nativeView.image = UIImage.imageWithCGImage(cgiImage_8);
                            _this.setAspect(_this.stretch);
                        });
                    }
                }
            });
        }
        else {
            this.nativeView.image = image;
            this.setAspect(this.stretch);
        }
    };
    ImageCacheIt.hasItem = function (src) {
        return new Promise(function (resolve, reject) {
            var manager = SDWebImageManager.sharedManager;
            if (manager) {
                var key = manager.cacheKeyForURL(NSURL.URLWithString(src));
                manager.imageCache.containsImageForKeyCacheTypeCompletion(key, 3, function (type) {
                    if (type > 0) {
                        resolve();
                    }
                    else {
                        reject();
                    }
                });
            }
            else {
                reject();
            }
        });
    };
    ImageCacheIt.deleteItem = function (src) {
        return new Promise(function (resolve, reject) {
            var manager = SDWebImageManager.sharedManager;
            if (manager) {
                var key = manager.cacheKeyForURL(NSURL.URLWithString(src));
                manager.imageCache.removeImageForKeyFromDiskWithCompletion(key, true, function () {
                    resolve();
                });
            }
            else {
                reject();
            }
        });
    };
    ImageCacheIt.clear = function () {
        return new Promise(function (resolve, reject) {
            var manager = SDWebImageManager.sharedManager;
            if (manager) {
                manager.clearMemory();
                manager.clearDiskOnCompletion(function () {
                    resolve();
                });
            }
        });
    };
    ImageCacheIt.getItem = function (src) {
        return new Promise(function (resolve, reject) {
            var manager = SDWebImageManager.sharedManager;
            if (manager) {
                if (src && src.indexOf('http') > -1) {
                    var nativeSrc = NSURL.URLWithString(src);
                    manager.loadImageWithURLOptionsProgressCompleted(nativeSrc, SDWebImageOptions.scaleDownLargeImages, function (receivedSize, expectedSize, path) {
                    }, function (image, data, error, type, finished, completedUrl) {
                        if (image === null && error !== null && data === null) {
                            reject(error.localizedDescription);
                        }
                        else if (finished && completedUrl != null) {
                            if (type === SDImageCacheType.disk) {
                                var key = manager.cacheKeyForURL(completedUrl);
                                var source = manager.imageCache.cachePathForKey(key);
                                resolve(source);
                            }
                            else {
                                var sharedCache = SDImageCache.sharedImageCache;
                                sharedCache.storeImageForKeyCompletion(image, completedUrl.absoluteString, function () {
                                    var key = manager.cacheKeyForURL(completedUrl);
                                    var source = manager.imageCache.cachePathForKey(key);
                                    resolve(source);
                                });
                            }
                        }
                    });
                }
            }
            else {
                reject();
            }
        });
    };
    ImageCacheIt.ciFilterMap = {};
    return ImageCacheIt;
}(image_cache_it_common_1.ImageCacheItBase));
exports.ImageCacheIt = ImageCacheIt;
//# sourceMappingURL=image-cache-it.ios.js.map